@@ -1,62 +1,42 @@
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE ImpredicativeTypes #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE CPP #-}
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Safe #-}
 #endif
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
+{-# LANGUAGE PolyKinds #-}
+#endif
 module Data.Dependent.Map.Internal where
 
 import Data.Dependent.Sum
 import Data.GADT.Compare
-import Data.GADT.Show
+import Data.Some
 #if MIN_VERSION_base(4,7,0)
 import Data.Typeable (Typeable)
 #endif
 
--- |A 'Key' is just a wrapper for the true key type @f@ which hides
--- the associated value type and presents the key's GADT-level 'GCompare' 
--- instance as a vanilla 'Ord' instance so it can be used in cases where we
--- don't care about the associated value.
-data Key f where Key :: !(f a) -> Key f
-instance GEq f => Eq (Key f) where
-    Key a == Key b = maybe False (const True) (geq a b)
-instance GCompare f => Ord (Key f) where
-    compare (Key a) (Key b) = weakenOrdering (gcompare a b)
-
-instance GShow f => Show (Key f) where
-    showsPrec p (Key k) = showParen (p>10)
-        ( showString "Key "
-        . gshowsPrec 11 k
-        )
-instance GRead f => Read (Key f) where
-    readsPrec p = readParen (p>10) $ \s ->
-        [ (withTag Key, rest')
-        | let (con, rest) = splitAt 4 s
-        , con == "Key "
-        , (withTag, rest') <- greadsPrec 11 rest
-        ]
-
--- |Dependent maps: f is a GADT-like thing with a facility for 
+-- |Dependent maps: 'k' is a GADT-like thing with a facility for 
 -- rediscovering its type parameter, elements of which function as identifiers
 -- tagged with the type of the thing they identify.  Real GADTs are one
--- useful instantiation of @f@, as are 'Tag's from "Data.Dependent.Tag".
+-- useful instantiation of @k@, as are 'Tag's from "Data.Unique.Tag" in the 
+-- 'prim-uniq' package.
 --
--- Semantically, @'DMap' f@ is equivalent to a set of @'DSum' f@ where no two
+-- Semantically, @'DMap' k f@ is equivalent to a set of @'DSum' k f@ where no two
 -- elements have the same tag.
 --
 -- More informally, 'DMap' is to dependent products as 'M.Map' is to @(->)@.
 -- Thus it could also be thought of as a partial (in the sense of \"partial
 -- function\") dependent product.
-data DMap k where
-    Tip :: DMap k
+data DMap k f where
+    Tip :: DMap k f
     Bin :: {- sz    -} !Int
         -> {- key   -} !(k v)
-        -> {- value -} v
-        -> {- left  -} !(DMap k)
-        -> {- right -} !(DMap k)
-        -> DMap k
+        -> {- value -} f v
+        -> {- left  -} !(DMap k f)
+        -> {- right -} !(DMap k f)
+        -> DMap k f
 #if MIN_VERSION_base(4,7,0)
     deriving Typeable
 #endif
@@ -69,14 +49,14 @@ data DMap k where
 --
 -- > empty      == fromList []
 -- > size empty == 0
-empty :: DMap k
+empty :: DMap k f
 empty = Tip
 
 -- | /O(1)/. A map with a single element.
 --
 -- > singleton 1 'a'        == fromList [(1, 'a')]
 -- > size (singleton 1 'a') == 1
-singleton :: k v -> v -> DMap k
+singleton :: k v -> f v -> DMap k f
 singleton k x = Bin 1 k x Tip Tip
 
 {--------------------------------------------------------------------
@@ -84,12 +64,12 @@ singleton k x = Bin 1 k x Tip Tip
 --------------------------------------------------------------------}
 
 -- | /O(1)/. Is the map empty?
-null :: DMap k -> Bool
+null :: DMap k f -> Bool
 null Tip    = True
 null Bin{}  = False
 
 -- | /O(1)/. The number of elements in the map.
-size :: DMap k -> Int
+size :: DMap k f -> Int
 size Tip                = 0
 size (Bin n _ _ _ _)    = n
 
@@ -97,10 +77,10 @@ size (Bin n _ _ _ _)    = n
 --
 -- The function will return the corresponding value as @('Just' value)@,
 -- or 'Nothing' if the key isn't in the map.
-lookup :: forall k v. GCompare k => k v -> DMap k -> Maybe v
+lookup :: forall k f v. GCompare k => k v -> DMap k f -> Maybe (f v)
 lookup k = k `seq` go
     where
-        go :: DMap k -> Maybe v
+        go :: DMap k f -> Maybe (f v)
         go Tip = Nothing
         go (Bin _ kx x l r) = 
             case gcompare k kx of
@@ -108,10 +88,10 @@ lookup k = k `seq` go
                 GGT -> go r
                 GEQ -> Just x
 
-lookupAssoc :: forall k v. GCompare k => Key k -> DMap k -> Maybe (DSum k)
-lookupAssoc (Key k) = k `seq` go
+lookupAssoc :: forall k f v. GCompare k => Some k -> DMap k f -> Maybe (DSum k f)
+lookupAssoc (This k) = k `seq` go
   where
-    go :: DMap k -> Maybe (DSum k)
+    go :: DMap k f -> Maybe (DSum k f)
     go Tip = Nothing
     go (Bin _ kx x l r) =
         case gcompare k kx of
@@ -131,7 +111,7 @@ lookupAssoc (Key k) = k `seq` go
     [balance k x l r] Restores the balance and size.
                       Assumes that the original tree was balanced and
                       that [l] or [r] has changed by at most one element.
-    [join k x l r]    Restores balance and size. 
+    [combine k x l r] Restores balance and size. 
 
   Furthermore, we can construct a new tree from two trees. Both operations
   assume that all values in [l] < all values in [r] and that [l] and [r]
@@ -141,7 +121,7 @@ lookupAssoc (Key k) = k `seq` go
     [merge l r]       Merges two trees and restores balance.
 
   Note: in contrast to Adam's paper, we use (<=) comparisons instead
-  of (<) comparisons in [join], [merge] and [balance]. 
+  of (<) comparisons in [combine], [merge] and [balance]. 
   Quickcheck (on [difference]) showed that this was necessary in order 
   to maintain the invariants. It is quite unsatisfactory that I haven't 
   been able to find out why this is actually the case! Fortunately, it 
@@ -149,19 +129,19 @@ lookupAssoc (Key k) = k `seq` go
 --------------------------------------------------------------------}
 
 {--------------------------------------------------------------------
-  Join 
+  Combine
 --------------------------------------------------------------------}
-join :: GCompare k => k v -> v -> DMap k -> DMap k -> DMap k
-join kx x Tip r  = insertMin kx x r
-join kx x l Tip  = insertMax kx x l
-join kx x l@(Bin sizeL ky y ly ry) r@(Bin sizeR kz z lz rz)
-  | delta*sizeL <= sizeR  = balance kz z (join kx x l lz) rz
-  | delta*sizeR <= sizeL  = balance ky y ly (join kx x ry r)
+combine :: GCompare k => k v -> f v -> DMap k f -> DMap k f -> DMap k f
+combine kx x Tip r  = insertMin kx x r
+combine kx x l Tip  = insertMax kx x l
+combine kx x l@(Bin sizeL ky y ly ry) r@(Bin sizeR kz z lz rz)
+  | delta*sizeL <= sizeR  = balance kz z (combine kx x l lz) rz
+  | delta*sizeR <= sizeL  = balance ky y ly (combine kx x ry r)
   | otherwise             = bin kx x l r
 
 
 -- insertMin and insertMax don't perform potentially expensive comparisons.
-insertMax,insertMin :: k v -> v -> DMap k -> DMap k
+insertMax,insertMin :: k v -> f v -> DMap k f -> DMap k f
 insertMax kx x t
   = case t of
       Tip -> singleton kx x
@@ -177,7 +157,7 @@ insertMin kx x t
 {--------------------------------------------------------------------
   [merge l r]: merges two trees.
 --------------------------------------------------------------------}
-merge :: DMap k -> DMap k -> DMap k
+merge :: DMap k f -> DMap k f -> DMap k f
 merge Tip r   = r
 merge l Tip   = l
 merge l@(Bin sizeL kx x lx rx) r@(Bin sizeR ky y ly ry)
@@ -189,7 +169,7 @@ merge l@(Bin sizeL kx x lx rx) r@(Bin sizeR ky y ly ry)
   [glue l r]: glues two trees together.
   Assumes that [l] and [r] are already balanced with respect to each other.
 --------------------------------------------------------------------}
-glue :: DMap k -> DMap k -> DMap k
+glue :: DMap k f -> DMap k f -> DMap k f
 glue Tip r = r
 glue l Tip = l
 glue l r   
@@ -201,7 +181,7 @@ glue l r
 -- > deleteFindMin (fromList [(5,"a"), (3,"b"), (10,"c")]) == ((3,"b"), fromList[(5,"a"), (10,"c")]) 
 -- > deleteFindMin                                            Error: can not return the minimal element of an empty map
 
-deleteFindMin :: DMap k -> (DSum k, DMap k)
+deleteFindMin :: DMap k f -> (DSum k f, DMap k f)
 deleteFindMin t 
   = case t of
       Bin _ k x Tip r -> (k :=> x ,r)
@@ -213,7 +193,7 @@ deleteFindMin t
 -- > deleteFindMax (fromList [(5,"a"), (3,"b"), (10,"c")]) == ((10,"c"), fromList [(3,"b"), (5,"a")])
 -- > deleteFindMax empty                                      Error: can not return the maximal element of an empty map
 
-deleteFindMax :: DMap k -> (DSum k, DMap k)
+deleteFindMax :: DMap k f -> (DSum k f, DMap k f)
 deleteFindMax t
   = case t of
       Bin _ k x l Tip -> (k :=> x,l)
@@ -255,7 +235,7 @@ delta,ratio :: Int
 delta = 4
 ratio = 2
 
-balance :: k v -> v -> DMap k -> DMap k -> DMap k
+balance :: k v -> f v -> DMap k f -> DMap k f -> DMap k f
 balance k x l r
   | sizeL + sizeR <= 1    = Bin sizeX k x l r
   | sizeR >= delta*sizeL  = rotateL k x l r
@@ -267,26 +247,26 @@ balance k x l r
     sizeX = sizeL + sizeR + 1
 
 -- rotate
-rotateL :: k v -> v -> DMap k -> DMap k -> DMap k
+rotateL :: k v -> f v -> DMap k f -> DMap k f -> DMap k f
 rotateL k x l r@(Bin _ _ _ ly ry)
   | size ly < ratio*size ry = singleL k x l r
   | otherwise               = doubleL k x l r
 rotateL _ _ _ Tip = error "rotateL Tip"
 
-rotateR :: k v -> v -> DMap k -> DMap k -> DMap k
+rotateR :: k v -> f v -> DMap k f -> DMap k f -> DMap k f
 rotateR k x l@(Bin _ _ _ ly ry) r
   | size ry < ratio*size ly = singleR k x l r
   | otherwise               = doubleR k x l r
 rotateR _ _ Tip _ = error "rotateR Tip"
 
 -- basic rotations
-singleL, singleR :: k v -> v -> DMap k -> DMap k -> DMap k
+singleL, singleR :: k v -> f v -> DMap k f -> DMap k f -> DMap k f
 singleL k1 x1 t1 (Bin _ k2 x2 t2 t3)  = bin k2 x2 (bin k1 x1 t1 t2) t3
 singleL _ _ _ Tip = error "singleL Tip"
 singleR k1 x1 (Bin _ k2 x2 t1 t2) t3  = bin k2 x2 t1 (bin k1 x1 t2 t3)
 singleR _ _ Tip _ = error "singleR Tip"
 
-doubleL, doubleR :: k v -> v -> DMap k -> DMap k -> DMap k
+doubleL, doubleR :: k v -> f v -> DMap k f -> DMap k f -> DMap k f
 doubleL k1 x1 t1 (Bin _ k2 x2 (Bin _ k3 x3 t2 t3) t4) = bin k3 x3 (bin k1 x1 t1 t2) (bin k2 x2 t3 t4)
 doubleL _ _ _ _ = error "doubleL"
 doubleR k1 x1 (Bin _ k2 x2 t1 (Bin _ k3 x3 t2 t3)) t4 = bin k3 x3 (bin k2 x2 t1 t2) (bin k1 x1 t3 t4)
@@ -295,7 +275,7 @@ doubleR _ _ _ _ = error "doubleR"
 {--------------------------------------------------------------------
   The bin constructor maintains the size of the tree
 --------------------------------------------------------------------}
-bin :: k v -> v -> DMap k -> DMap k -> DMap k
+bin :: k v -> f v -> DMap k f -> DMap k f -> DMap k f
 bin k x l r
   = Bin (size l + size r + 1) k x l r
 
@@ -321,20 +301,20 @@ bin k x l r
   values between the range [lo] to [hi]. The returned tree is either
   empty or the key of the root is between @lo@ and @hi@.
 --------------------------------------------------------------------}
-trim :: (Key k -> Ordering) -> (Key k -> Ordering) -> DMap k -> DMap k
+trim :: (Some k -> Ordering) -> (Some k -> Ordering) -> DMap k f -> DMap k f
 trim _     _     Tip = Tip
 trim cmplo cmphi t@(Bin _ kx _ l r)
-  = case cmplo (Key kx) of
-      LT -> case cmphi (Key kx) of
+  = case cmplo (This kx) of
+      LT -> case cmphi (This kx) of
               GT -> t
               _  -> trim cmplo cmphi l
       _  -> trim cmplo cmphi r
               
-trimLookupLo :: GCompare k => Key k -> (Key k -> Ordering) -> DMap k -> (Maybe (DSum k), DMap k)
+trimLookupLo :: GCompare k => Some k -> (Some k -> Ordering) -> DMap k f -> (Maybe (DSum k f), DMap k f)
 trimLookupLo _  _     Tip = (Nothing,Tip)
 trimLookupLo lo cmphi t@(Bin _ kx x l r)
-  = case compare lo (Key kx) of
-      LT -> case cmphi (Key kx) of
+  = case compare lo (This kx) of
+      LT -> case cmphi (This kx) of
               GT -> (lookupAssoc lo t, t)
               _  -> trimLookupLo lo cmphi l
       GT -> trimLookupLo lo cmphi r
@@ -345,20 +325,20 @@ trimLookupLo lo cmphi t@(Bin _ kx x l r)
   [filterGt k t] filter all keys >[k] from tree [t]
   [filterLt k t] filter all keys <[k] from tree [t]
 --------------------------------------------------------------------}
-filterGt :: GCompare k => (Key k -> Ordering) -> DMap k -> DMap k
+filterGt :: GCompare k => (Some k -> Ordering) -> DMap k f -> DMap k f
 filterGt cmp = go
   where
     go Tip              = Tip
-    go (Bin _ kx x l r) = case cmp (Key kx) of
-              LT -> join kx x (go l) r
+    go (Bin _ kx x l r) = case cmp (This kx) of
+              LT -> combine kx x (go l) r
               GT -> go r
               EQ -> r
 
-filterLt :: GCompare k => (Key k -> Ordering) -> DMap k -> DMap k
+filterLt :: GCompare k => (Some k -> Ordering) -> DMap k f -> DMap k f
 filterLt cmp = go
   where
     go Tip              = Tip
-    go (Bin _ kx x l r) = case cmp (Key kx) of
+    go (Bin _ kx x l r) = case cmp (This kx) of
           LT -> go l
-          GT -> join kx x l (go r)
+          GT -> combine kx x l (go r)
           EQ -> l
